@@ -29,6 +29,7 @@ import logging
 import threading
 
 from . import concurrent
+from . import utils
 
 
 class NotRunning(Exception):
@@ -74,7 +75,7 @@ class Executor(object):
     _log = logging.getLogger('Executor')
 
     def __init__(self, name, workers_count, max_tasks, scheduler,
-                 max_workers=None):
+                 max_workers=None, log=None):
         """
         :param name: Name of the executor; no special purpose, just for
           logging and debugging.
@@ -96,6 +97,9 @@ class Executor(object):
           it is not None and it gets reached then no further workers are
           created.
         :type max_workers: int or None
+        :param log: logger instance to override the default logger. This is
+          useful for testing
+        :type log: logger as returned by logging.getLogger()
 
         """
         self._name = name
@@ -104,6 +108,8 @@ class Executor(object):
         self._worker_id = 0
         self._tasks = TaskQueue(max_tasks)
         self._scheduler = scheduler
+        if log is not None:
+            self._log = log
         self._workers = set()
         self._lock = threading.Lock()
         self._running = False
@@ -132,7 +138,7 @@ class Executor(object):
         for worker in workers:
             worker.join()
 
-    def dispatch(self, callable, timeout=None):
+    def dispatch(self, callable, timeout=None, discard=True):
         """
         Dispatches a new task to the executor.
 
@@ -145,7 +151,7 @@ class Executor(object):
         """
         if not self._running:
             raise NotRunning()
-        self._tasks.put(Task(callable, timeout))
+        self._tasks.put(Task(callable, timeout, discard))
 
     # Serving workers
 
@@ -227,7 +233,8 @@ class Executor(object):
     def _add_worker(self):
         name = "%s/%d" % (self.name, self._worker_id)
         self._worker_id += 1
-        worker = _Worker(self, self._scheduler, name)
+        worker = _Worker(self, self._scheduler, name, self._log)
+        worker.start()
         self._workers.add(worker)
 
 
@@ -242,22 +249,27 @@ class _Worker(object):
 
     _log = logging.getLogger('Executor')
 
-    def __init__(self, executor, scheduler, name):
+    def __init__(self, executor, scheduler, name, log=None):
         self._executor = executor
         self._scheduler = scheduler
         self._discarded = False
         self._task_counter = 0
         self._lock = threading.Lock()
+        if log is not None:
+            self._log = log
         self._thread = concurrent.thread(self._run, name=name,
                                          logger=self._log.name)
-        self._log.debug('Starting worker %s' % name)
-        self._thread.start()
         self._task = None
+        self._scheduled_check = None
 
     @property
     def name(self):
         return self._thread.name
 
+    def start(self):
+        self._log.debug('Starting worker %s' % self.name)
+        self._thread.start()
+
     def join(self):
         self._log.debug('Waiting for worker %s', self.name)
         self._thread.join()
@@ -280,10 +292,11 @@ class _Worker(object):
 
     def _execute_task(self):
         task = self._executor._next_task()
-        discard = self._discard_after(task.timeout)
+        with self._lock:
+            self._scheduled_check = self._check_after(task.timeout)
         self._task = task
         try:
-            task.callable()
+            task()
         except Exception:
             self._log.exception("Unhandled exception in %s", task)
         finally:
@@ -293,31 +306,40 @@ class _Worker(object):
             # blocked on callable when we discard it or it just finished.
             # However, we expect that most of times only blocked threads
             # will be discarded.
-            if discard is not None:
-                discard.cancel()
             with self._lock:
+                if self._scheduled_check is not None:
+                    self._scheduled_check.cancel()
+                    self._scheduled_check = None
                 self._task_counter += 1
             if self._discarded:
                 raise _WorkerDiscarded()
 
-    def _discard_after(self, timeout):
+    def _check_after(self, timeout):
         if timeout is not None:
-            discard = functools.partial(self._discard, self._task_counter)
-            return self._scheduler.schedule(timeout, discard)
+            check_task = functools.partial(
+                self._check_task, self._task_counter)
+            return self._scheduler.schedule(timeout, check_task)
         return None
 
-    def _discard(self, task_number):
+    def _check_task(self, task_number):
         with self._lock:
             if task_number != self._task_counter:
                 return
-            if self._discarded:
-                raise AssertionError("Attempt to discard worker twice")
-            self._discarded = True
-        # Please make sure the executor call is performed outside the lock --
-        # there is another lock involved in the executor and we don't want to
-        # fall into a deadlock incidentally.
-        self._executor._worker_discarded(self)
-        self._log.debug("Worker discarded: %s", self)
+            if self._task.discard:
+                if self._discarded:
+                    raise AssertionError("Attempt to discard worker twice")
+                self._discarded = True
+            else:
+                self._scheduled_check = self._check_after(self._task.timeout)
+        if self._discarded:
+            # Please make sure the executor call is performed outside the lock
+            # -- there is another lock involved in the executor and we don't
+            # want to fall into a deadlock incidentally.
+            self._executor._worker_discarded(self)
+            self._log.info("Worker discarded: %s", self)
+        else:
+            # we want to avoid to log with the lock held, so we do it here.
+            self._log.warning("Worker blocked: %s", self)
 
     def __repr__(self):
         return "<Worker name=%s %s%s task#=%s at 0x%x>" % (
@@ -329,7 +351,32 @@ class _Worker(object):
         )
 
 
-Task = collections.namedtuple("Task", "callable, timeout")
+class Task(object):
+
+    def __init__(self, callable, timeout, discard=True):
+        self._callable = callable
+        self.timeout = timeout
+        self.discard = discard
+        self._start = None
+
+    @property
+    def duration(self):
+        if self._start is None:
+            return 0
+        return utils.monotonic_time() - self._start
+
+    def __call__(self):
+        self._start = utils.monotonic_time()
+        self._callable()
+
+    def __repr__(self):
+        return "<Task %s%s timeout=%s, duration=%d at 0x%x>" % (
+            "discardable " if self.discard else "",
+            self._callable,
+            self.timeout,
+            self.duration,
+            id(self)
+        )
 
 
 class TaskQueue(object):
